/*
 * File:  BitWise.hh
 * Author:  mikolas
 * Created on:  Wed Aug 1 16:31:57 WEST 2012
 * Copyright (C) 2012, Mikolas Janota
 */
#ifndef BITWISE_HH_17485
#define BITWISE_HH_17485
#include <iostream>
#include <vector>
#include "MiFuMaXTypes.hh"
using std::vector;
using Mifumax::ClauseVector;
using Minisat::Lit;
using Minisat::mkLit;
using Minisat::Var;
using Minisat::vec;

class BitWise {
public:
  BitWise(ClauseVector& clauses, const vector<Var>& pvars, Var _max_id)
  : clauses(clauses)
  , vars(pvars)
  , max_id(_max_id)
  {
    initialize();
  }

  Var                  get_max_id() { return max_id; }
  void                 encode();
private:
  ClauseVector&        clauses;
  const vector<Var>&   vars; 
  Var                  max_id;
  vector<Var>          aux;
  void                 initialize();

  inline Var b(size_t i) const { 
    assert(i<aux.size());
    return aux[i];
  }

  inline Var v(size_t v_index) const { 
    assert(v_index>=0);
    assert(v_index<vars.size());
    return vars[v_index];
   }

  inline  void    addClause (Lit p);
  inline  void    addClause (Lit p, Lit q);
  inline  void    addClause (Lit p, Lit q, Lit r);
  inline  vector<Lit>&  addClause ();
};

inline  vector<Lit>&  BitWise::addClause() {
  const size_t oldsz=clauses.size();
  clauses.resize(oldsz+1);
  return clauses[oldsz]; 
}

inline void BitWise::addClause (Lit p) {
  const size_t oldsz=clauses.size();
  clauses.resize(oldsz+1);
  clauses[oldsz].push_back(p);
}

inline void BitWise::addClause (Lit p, Lit q) { 
  const size_t oldsz=clauses.size();
  clauses.resize(oldsz+1);
  clauses[oldsz].push_back(p);
  clauses[oldsz].push_back(q);
}

inline void BitWise::addClause (Lit p, Lit q, Lit r) { 
  const size_t oldsz=clauses.size();
  clauses.resize(oldsz+1);
  clauses[oldsz].push_back(p);
  clauses[oldsz].push_back(q);
  clauses[oldsz].push_back(r);
}
#endif /* BITWISE_HH_17485 */
