/*
 * File:  ReadWCNFBase.cc
 * Author:  mikolas
 * Created on:  Thu, Jun 28, 2012 10:07:45 AM
 * Copyright (C) 2012, Mikolas Janota
 */
#include "ReadWCNFBase.hh"
#include "Reader.hh"
#include "MiFuMaXTypes.hh"
#include <vector>
#include <string>
#include <ctype.h>
#include <assert.h>
using Minisat::lit_Undef;
using namespace Mifumax;

ReadWCNFBase::ReadWCNFBase(Reader& input_file)
  : in(input_file), mxid(0), top(-1), header_read(false)
{}

ReadWCNFBase::ReadWCNFBase(const ReadWCNFBase& orig)
: in(orig.in), mxid(orig.mxid)
{assert(false);}

ReadWCNFBase::~ReadWCNFBase() {}

void ReadWCNFBase::read_cnf_clause(vector<Lit>& lits, Reader& in, Var& mxid) {
  Lit parsed_lit;
  lits.clear();
  for (;;){
    in.skip_whitespace();
    parsed_lit = parse_lit(in);
    if (parsed_lit == lit_Undef) break;
    const Var v = var(parsed_lit);
    if (v > mxid) { mxid = v; }
    lits.push_back(parsed_lit);
  }
}

void ReadWCNFBase::read_cnf_clause_into(ClauseVector& into) {
  const size_t oldsz=into.size();
  into.resize(oldsz+1);
  read_cnf_clause(into[oldsz],in,mxid);
}


void ReadWCNFBase::read() {
  vector<Lit> literals;
  for (;;){
    in.skip_whitespace();
    if (*in == EOF)
      break;
    else if (*in == 'c') { in.skip_line(); }
    else if (*in == 'p') {
      read_header();
      header_read = true;
    } else {
      parse_clause();
    }
  }
}

void ReadWCNFBase::read_header() {
  assert(*in=='p');
  ++in;
  in.skip_whitespace();
  if (*in=='w') {
    is_weighted=true;
    ++in;
  } else  {
    is_weighted=false;
    is_partial=false;
  }

  if (!in.match("cnf")) {
     throw ReadCNFException("cnf/wcnf expected in the problem line");
  }
  in.skip_true_whitespace();
  r_max_id=parse_header_const(in);
  in.skip_true_whitespace();
  r_max_cls=parse_header_const(in);
  if (is_weighted) {
    in.skip_true_whitespace();
    if (*in=='\n') {
      ++in;
      is_partial=false;
    } else {
      top=parse_weight(in);
      is_partial=true;
    }
  }
  assert(!is_partial||is_weighted);
}

Lit ReadWCNFBase::parse_lit(Reader& in) {
    Var  v = 0;
    bool neg = false;
    if (*in=='-') { neg=true; ++in; }
    else if (*in == '+') ++in;
    if ((*in < '0') || (*in > '9')) {
        string s("unexpected char in place of a literal: ");  s+=*in;
        throw ReadCNFException(s);
    }
    while (*in >= '0' && *in <= '9') {
        v = v*10 + (*in - '0');
        ++in;
    }
    return v ? mkLit(v, neg) : lit_Undef;
}

WeightType ReadWCNFBase::parse_weight(Reader& in) {
  WeightType w = 0;
  if ((*in < '0') || (*in > '9')) {
    string s("unexpected char in place of weight: ");  s+=*in;
    throw ReadCNFException(s);
  }
  while (*in >= '0' && *in <= '9') {
    w = w*10 + (*in - '0');
    ++in;
  }
  return w;
}

size_t ReadWCNFBase::parse_header_const(Reader& in) {
  size_t c = 0;
  if ((*in < '0') || (*in > '9')) {
    string s("unexpected char in place of weight: ");  s+=*in;
    throw ReadCNFException(s);
  }
  while (*in >= '0' && *in <= '9') {
    c = c*10 + (*in - '0');
    ++in;
  }
  return c;
}

