/*
 * File:  ReadWCNFBase.hh
 * Author:  mikolas
 * Created on:  Thu, Jun 28, 2012 10:07:42 AM
 * Copyright (C) 2012, Mikolas Janota
 */
#ifndef READWCNF_HH_4048
#define READWCNF_HH_4048
#include <zlib.h>
#include "Reader.hh"
#include "ReadCNFException.hh"
#include "MiFuMaXTypes.hh"
using std::vector;
using Mifumax::ClauseVector;
using Minisat::Lit;
using Minisat::mkLit;
using Minisat::Var;
using std::string;

namespace Mifumax {
class ReadWCNFBase {
public:
  ReadWCNFBase(Reader& input_file);
  ReadWCNFBase(const ReadWCNFBase& orig);
  virtual ~ReadWCNFBase();
  void                       read();
  Var                        get_max_id() const         {return mxid;}
  bool                       get_header_read() const {return header_read;}
  static Lit                 parse_lit(Reader& in);
  static WeightType          parse_weight(Reader& in);
  static size_t              parse_header_const(Reader& in);
  static void                read_cnf_clause(vector<Lit>& lits, Reader& in, Var& mxid);
protected:
  Reader&           in;
  Var               mxid;
  bool              is_weighted;
  bool              is_partial;
  WeightType        top;
  bool              header_read;
  size_t            r_max_cls; // number of as declared in the header
  size_t            r_max_id;  // max id as declared in the header
  void              read_cnf_clause(vector<Lit>& lits);
  void              read_cnf_clause_into(ClauseVector& into);
  void              read_header();
  virtual void      parse_clause() = 0;
};
}
#endif /* READWCNF_HH_4048 */
