/*
 * File:  SeqCounter.hh
 * Author:  mikolas
 * Created on:  Thu Dec 1 15:53:16 GMTST 2011
 * Copyright (C) 2011, Mikolas Janota
 */
#ifndef SEQCOUNTER_HH_25162
#define SEQCOUNTER_HH_25162
#define SEQCOUNTER_DBG(t)
#include <iostream>
#include <vector>
#include "MiFuMaXTypes.hh"
using std::vector;
using Mifumax::ClauseVector;
using Minisat::Lit;
using Minisat::mkLit;
using Minisat::Var;
using Minisat::vec;

class SeqCounter {
public:
  SeqCounter(ClauseVector& clauses, const vector<Var>& pvars, size_t ptval, Var _max_id)
  : clauses(clauses)
  , vars(pvars)
  , tval(ptval)
  , max_id(_max_id)
  {
    for (size_t i=0; i<(vars.size()-1)*tval; ++i) aux.push_back(++max_id);
  }

  Var                  get_max_id() { return max_id; }
  void                 encode();
  void                 encode_all0();
private:
  ClauseVector&        clauses;
  const vector<Var>&   vars; 
  const size_t         tval;
  Var                  max_id;
  vector<Var>          aux;

  inline Var s(size_t v_index, size_t j) { 
    assert(v_index>0);
    assert(v_index<vars.size());
    assert(j>=1);
    assert(j<=tval);
    return aux[(v_index-1)*tval + j-1];
  }

  inline Var v(size_t v_index) { 
    assert(v_index>0);
    assert(v_index<=vars.size());
    return vars[v_index-1];
   }

  inline  void    addClause (Lit p);
  inline  void    addClause (Lit p, Lit q);
  inline  void    addClause (Lit p, Lit q, Lit r);
};

inline void SeqCounter::addClause (Lit p) {
  const size_t oldsz=clauses.size();
  clauses.resize(oldsz+1);
  clauses[oldsz].push_back(p);
}

inline void SeqCounter::addClause (Lit p, Lit q) { 
  const size_t oldsz=clauses.size();
  clauses.resize(oldsz+1);
  clauses[oldsz].push_back(p);
  clauses[oldsz].push_back(q);
}

inline void SeqCounter::addClause (Lit p, Lit q, Lit r) { 
  const size_t oldsz=clauses.size();
  clauses.resize(oldsz+1);
  clauses[oldsz].push_back(p);
  clauses[oldsz].push_back(q);
  clauses[oldsz].push_back(r);
}
#endif /* SEQCOUNTER_HH_25162 */
