/******************************************************************************\
 *    This file is part of packup.                                            *
 *                                                                            *
 *    packup is free software: you can redistribute it and/or modify          *
 *    it under the terms of the GNU General Public License as published by    *
 *    the Free Software Foundation, either version 3 of the License, or       *
 *    (at your option) any later version.                                     *
 *                                                                            *
 *    packup is distributed in the hope that it will be useful,               *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *    GNU General Public License for more details.                            *
 *                                                                            *
 *    You should have received a copy of the GNU General Public License       *
 *    along with packup.  If not, see <http://www.gnu.org/licenses/>.         *
\******************************************************************************/
/*
 * File:  main.cc
 * Author:  mikolas
 * Created on:  Mon, Apr 02, 2012 3:18:18 PM
 * Copyright (C) 2012, Mikolas Janota
 */
#include <sstream>
#include <string>
#include <iostream>
#include <sstream>
#include "ReadWeighted.hh"
#include "ReadPartial.hh"
#include "MiFuMaX.hh"
#include "MiFuMaXWeighted.hh"
using std::string;
using std::stringstream;
using std::cout;
using std::cerr;
using std::endl;
using namespace Mifumax;

void  t();void  t1();void  t2();void  t3();void  t4();void  t5();
void print_result(const bool result, const MiFuMaX& solver);
void print_result(const bool result, const MiFuMaXWeighted& solver);
int run_weighted(int argc, char** argv);
int run_partial(int argc, char** argv);

int main(int argc, char** argv) {
  cout<<"c MiFuMax MaxSAT solver"<<endl;
  cout<<"c Mikolas Janota, (C) 2013"<<endl;
  if (argc<=1) {
    cout<<"no file given, test mode (use filename '-' to read stdin)" <<endl;
    t();
    return 0;
  }

#ifdef PARTIAL
  return run_partial(argc, argv);
#else
  return run_weighted(argc, argv);
#endif
}


int run_weighted(int argc, char** argv) {
  const string flafile(argv[1]);
  gzFile ff=Z_NULL;
  Reader* fr;

  if (flafile.size()==1 && flafile[0]=='-') {
    fr=new Reader(std::cin);
  } else {
    ff = gzopen(flafile.c_str(), "rb");
    if (ff== Z_NULL) {
      cerr << "ERROR: Unable to open file: " << flafile << endl;
      exit(100);
    }
    fr = new Reader(ff);
  }

  ReadWeighted rwcnf(*fr);
  try {
    rwcnf.read();
  } catch (ReadCNFException& rex) {
    cerr << "ERROR: " << rex.what() << endl;
    exit(100);
  }

  cout << "c weighted MaxSAT" << endl;
  if (rwcnf.get_hard_clause_vector().empty()) {
    cout << "c no hard clauses MaxSAT" << endl;
  } else {
    cout << "c some hard clauses (partial MaxSAT)" << endl;
  }

  if (!rwcnf.get_header_read())
    cerr << "WARNING: Missing problem line in the input file." << endl;


  MiFuMaXWeighted s(rwcnf.get_max_id(), rwcnf.get_hard_clause_vector(), rwcnf.get_soft_clause_vector());
  const bool result = s.solve();
  print_result(result,s);
  assert(rwcnf.get_hard_clause_vector().size() || result); // MaxSat has to always be satisfiable
  return 0;
}

int run_partial(int argc, char** argv) {
  const string flafile(argv[1]);
  gzFile ff=Z_NULL;
  Reader* fr;

  if (flafile.size()==1 && flafile[0]=='-') {
    fr=new Reader(std::cin);
  } else {
    ff = gzopen(flafile.c_str(), "rb");
    if (ff== Z_NULL) {
      cerr << "ERROR: Unable to open file: " << flafile << endl;
      exit(100);
    }
    fr = new Reader(ff);
  }


  ReadPartial rwcnf(*fr);
  try {
    rwcnf.read();
  } catch (ReadCNFException& rex) {
    cerr << "ERROR: " << rex.what() << endl;
    exit(100);
  }

  cout << "c unweighted MaxSAT" << endl;
  if (rwcnf.get_hard_clause_vector().empty()) {
    cout << "c no hard clauses MaxSAT" << endl;
  } else {
    cout << "c some hard clauses (partial MaxSAT)" << endl;
  }

  if (!rwcnf.get_header_read())
    cerr << "WARNING: Missing problem line in the input file." << endl;


  MiFuMaX s(rwcnf.get_max_id(), rwcnf.get_hard_clause_vector(),rwcnf.get_soft_clause_vector());
  const bool result = s.solve();
  print_result(result,s);
  assert(rwcnf.get_hard_clause_vector().size() || result); // MaxSat has to always be satisfiable
  return 0;
}


void print_result(const bool result, const MiFuMaX& solver) {
  if (result) {
    cout<<"o "<<solver.get_optimum()<<endl;
    cout<<"s OPTIMUM FOUND"<<endl;
    vec<lbool> model;
    solver.get_solution(model);
    cout<<"v";
    for (Var v=1; v<model.size(); ++v) {
      if (model[v]==l_Undef) continue;
      cout<<" ";
      if (model[v]==l_False) cout<<'-';
      cout<<v;
    }
    cout<<endl;
  } else {
    cout<<"s UNSATISFIABLE"<<endl;
  }
}

void print_result(const bool result, const MiFuMaXWeighted& solver) {
  if (result) {
    cout<<"o "<<solver.get_optimum()<<endl;
    cout<<"s OPTIMUM FOUND"<<endl;
    vec<lbool> model;
    solver.get_solution(model);
    cout<<"v";
    for (Var v=1; v<model.size(); ++v) {
      if (model[v]==l_Undef) continue;
      cout<<" ";
      if (model[v]==l_False) cout<<'-';
      cout<<v;
    }
    cout<<endl;
  } else {
    cout<<"s UNSATISFIABLE"<<endl;
  }
}

/*************************** TESTING *********************************/

void t1() {
  ClauseVector cnf(2);
  cnf[0].push_back(~mkLit(1));
  cnf[1].push_back( mkLit(1));
  MiFuMaX s(1, cnf);
  const bool result = s.solve();
  cout << result << endl;
}

void t2() {
  ClauseVector cnf(3);
  cnf[0].push_back(~mkLit(1));
  cnf[1].push_back( mkLit(1));
  cnf[2].push_back( mkLit(1) );
  cnf[2].push_back( mkLit(2) );
  MiFuMaX s(2, cnf);
  const bool result = s.solve();
  cout << result << endl;
}


void t3() {
  ClauseVector cnf(6);
  cnf[0].push_back(~mkLit(1));
  cnf[0].push_back( mkLit(2));
  cnf[1].push_back(~mkLit(1) );
  cnf[1].push_back( mkLit(3) );
  cnf[2].push_back(~mkLit(2) );
  cnf[2].push_back(~mkLit(3) );
  cnf[3].push_back( mkLit(1));
  cnf[4].push_back(~mkLit(4));
  cnf[5].push_back( mkLit(4));
  MiFuMaX s(4, cnf);
  const bool result = s.solve();
  cout << result << endl;
}

void t4() {
  cout<<"t4:"<<endl;
  ClauseVector hard_cnf(1);
  ClauseVector soft_cnf(2);
  hard_cnf[0].push_back(~mkLit(1));
  hard_cnf[0].push_back(~mkLit(2));
  soft_cnf[0].push_back( mkLit(1));
  soft_cnf[1].push_back( mkLit(2));
  MiFuMaX s(2, hard_cnf, soft_cnf);
  const bool result = s.solve();
  print_result(result,s);
}


void t5() {
  cout<<"t5:"<<endl;
  ClauseVector hard_cnf(2);
  ClauseVector soft_cnf;
  hard_cnf[0].push_back(~mkLit(1));
  hard_cnf[1].push_back( mkLit(1));
  MiFuMaX s(1, hard_cnf, soft_cnf);
  const bool result = s.solve();
  print_result(result,s);
}


void t() {
  t1();
  t2();
  t3();
  t4();
  t5();
}

